/***********************************************************************************
* T-SQL: Simple Changes That Go a Long Way
* Dave Valentine
* Window Functions Demo SQL Script
***********************************************************************************/

USE AdventureWorks2012;
GO

--SET STATISTICS IO ON so we can see disk activity
SET STATISTICS IO ON;
GO


/***********************************************************************************
* PART I
*
* Running the following query finds the TOP 3  Sales per person Persons by Year.

FirstName	  LastName	   TotalDue
Syed		  Abbas		   65910.6732
Syed		  Abbas		   35912.8785
Syed		  Abbas		   31019.4824
Amy		  Alberts		   81029.9462
Amy		  Alberts		   80819.7335
Amy		  Alberts		   77823.9958
Pamela	  Ansman-Wolfe	   105493.6317
Pamela	  Ansman-Wolfe	   103227.0017
Pamela	  Ansman-Wolfe	   99023.6736
***********************************************************************************/
SELECT TOP 3 p.FirstName
           , p.LastName
           , sp.SalesYTD
FROM sales.SalesPerson sp
     JOIN HumanResources.Employee e ON e.BusinessEntityID = sp.BusinessEntityID
     JOIN Person.Person p ON p.BusinessEntityID = e.BusinessEntityID
ORDER BY SalesYTD DESC;  
GO



/***********************************************************************************
* Running the following query finds the TOP 3 Sales for a Sales Person.
***********************************************************************************/
SELECT TOP 3 FirstName
           , LastName
           , TotalDue
FROM Sales.SalesOrderHeader soh
JOIN HumanResources.Employee e ON e.BusinessEntityID = soh.SalesPersonID
JOIN Person.Person p ON p.BusinessEntityID = e.BusinessEntityID
ORDER BY SalesPersonID , TotalDue DESC;
GO



/***********************************************************************************
* Running the following query finds the TOP 3 sales for each Sales Person.
* This script uses a bit of TSQL trickery. By using a COUNT and a correlated sub 
* query we can get a row number for each Sales Person's totals.  Using that count 
* we can then limit the results to the top 3.
***********************************************************************************/
SELECT FirstName
     , LastName
     , TotalDue
FROM(
     SELECT SalesOrderID
          , SalesPersonID
          , TotalDue
          , TopRankingSales = (
			  SELECT COUNT(DISTINCT TotalDue)
			  FROM Sales.SalesOrderHeader s
			  WHERE SalesPersonID = so.SalesPersonID
			    AND TotalDue > so.TotalDue) 
     FROM Sales.SalesOrderHeader so
     WHERE so.SalesPersonID IS NOT NULL) AS soh
JOIN HumanResources.Employee e ON e.BusinessEntityID = soh.SalesPersonID
JOIN Person.Person p ON p.BusinessEntityID = e.BusinessEntityID
WHERE TopRankingSales < 3
ORDER BY LastName, FirstName, TotalDue DESC;
GO




/***********************************************************************************
* Running the following query finds the TOP 3 sales for each Sales Person.
* This script uses a ranking window function to order a Sales Persons sales by the 
* total due.  Using this ranking value we can limit to the top 3.
***********************************************************************************/
SELECT FirstName
     , LastName
     , TotalDue
FROM(
     SELECT FirstName
          , LastName
          , TotalDue
          , TopRankingSales = RANK() OVER (PARTITION BY SalesPersonID ORDER BY TotalDue DESC)
     FROM Sales.SalesOrderHeader soh
     JOIN HumanResources.Employee e ON e.BusinessEntityID = soh.SalesPersonID
     JOIN Person.Person p ON p.BusinessEntityID = e.BusinessEntityID) AS RankingSalesOrder
WHERE TopRankingSales <= 3
ORDER BY LastName, FirstName, TotalDue DESC;
GO













/***********************************************************************************
* PART II
*
* Running the following query returns the previous, current, and next totals for
* a sales order.

SalesOrderID	 OrderDate			PreviousTotal	 CurrentTotal	  NextTotal
52626		 2007-08-16 00:00:00.000	NULL			 41.9569		  59.659
53834		 2007-09-05 00:00:00.000	41.9569		 59.659		  53.0069
54332		 2007-09-14 00:00:00.000	59.659		 53.0069		  9.934
56910		 2007-10-30 00:00:00.000	53.0069		 9.934		  8.0444
57639		 2007-11-09 00:00:00.000	9.934		 8.0444		  164.6229
58600		 2007-11-26 00:00:00.000	8.0444		 164.6229		  NULL
***********************************************************************************/
SELECT soh.SalesOrderID
     , soh.OrderDate
     , PreviousTotal = soh1.TotalDue
     , CurrentTotal  = soh.TotalDue
     , NextTotal     = soh2.TotalDue
FROM Sales.SalesOrderHeader soh
LEFT JOIN Sales.SalesOrderHeader soh1 ON soh1.SalesOrderID = soh.SalesOrderID - 1 
LEFT JOIN Sales.SalesOrderHeader soh2 ON soh2.SalesOrderID = soh.SalesOrderID + 1;
GO

/***********************************************************************************
* Running the following query returns the previous, current, and next totals for
* a sales order for a given CustomerID.  Notice that previous and next are all null?
***********************************************************************************/
DECLARE @CustomerID int = 11019;
SELECT soh.SalesOrderID
     , soh.OrderDate
     , PreviousTotal = soh1.TotalDue
     , CurrentTotal = soh.TotalDue
     , NextTotal = soh2.TotalDue
FROM Sales.SalesOrderHeader soh
LEFT JOIN Sales.SalesOrderHeader soh1 ON soh1.SalesOrderID = soh.SalesOrderID - 1
                                      AND soh1.CustomerID = soh.CustomerID
LEFT JOIN Sales.SalesOrderHeader soh2 ON soh2.SalesOrderID = soh.SalesOrderID + 1
                                      AND soh2.CustomerID = soh.CustomerID
WHERE soh.CustomerID = @CustomerID;
GO

/***********************************************************************************
* With SQL Server 2012 we have two new analytical functions LAG and LEAD.  These
* functions allow us to partition the results by CustomerID and get the results
* that the prior query was missing.  Running the following query returns the 
* previous, current, and next totals for a sales order for a given CustomerID.
***********************************************************************************/
DECLARE @CustomerID int = 11019;
SELECT SalesOrderID
     , OrderDate
     , PreviousTotal = lag(TotalDue)  OVER (PARTITION BY CustomerID ORDER BY OrderDate , SalesOrderID)
     , CurrentTotal  = TotalDue
     , NextTotal     = lead(TotalDue) OVER (PARTITION BY CustomerID ORDER BY OrderDate , SalesOrderID)
FROM Sales.SalesOrderHeader soh
WHERE soh.CustomerID = @CustomerID;

/* END Window Functions DEMO */