/***********************************************************************************
* T-SQL: Simple Changes That Go a Long Way
* Dave Valentine
* Common Table Expression Demo SQL Script
***********************************************************************************/
USE AdventureWorks2012;
GO

--SET STATISTICS IO ON so we can see disk activity
SET STATISTICS IO ON;
GO

/***********************************************************************************
* PART I
*
* This DEMO is based off a blog post that Rick Krueger created.
* http://www.dataogre.com/2012/06/18/remove-duplicate-rows-using-a-cte/
* 
* First create some duplicate records
***********************************************************************************/
INSERT INTO Sales.SpecialOffer
         (Description, DiscountPct, Type, Category, StartDate, EndDate
        , MinQty, MaxQty, rowguid, ModifiedDate)
SELECT Description
     , DiscountPct
     , Type
     , Category
     , StartDate
     , EndDate
     , MinQty
     , MaxQty
     , rowguid = NEWID()
     , ModifiedDate
FROM Sales.SpecialOffer so;
GO

/***********************************************************************************
* Query to show duplicate records in the Sales.SpecialOffer table.
***********************************************************************************/
SELECT   Description , DiscountPct
       , Type
       , Category
       , RecordCount = COUNT(1)
FROM     Sales.SpecialOffer so
GROUP BY Description
       , DiscountPct
       , Type
       , Category
HAVING   COUNT(1) > 1;
GO


/***********************************************************************************
* Using a CTE (Common Table Expression) with a Ranking function ROW_NUMBER, we
* can add a row number to each duplicate record and order by SpecialOfferID.  Any
* record with a RowNumber greater than 1 is a duplicate
***********************************************************************************/
WITH DeDup
    AS (SELECT RowNumber = ROW_NUMBER() OVER (PARTITION BY Description, DiscountPct, Type, Category 
                                              ORDER BY SpecialOfferID)
             , Description
             , DiscountPct
             , Type
             , Category
		   , SpecialOfferID
        FROM Sales.SpecialOffer so)
    SELECT RowNumber
         , Description
         , DiscountPct
         , Type
         , Category
	    , SpecialOfferID
    FROM DeDup;
GO


/***********************************************************************************
* Using the RowNumber found using the ranking function in the CTE we can delete
* all records that have a RowNumber greater than 1
***********************************************************************************/
WITH  DeDup
    AS (SELECT RowNumber = ROW_NUMBER() OVER (PARTITION BY Description, DiscountPct, Type, Category 
                                              ORDER BY SpecialOfferID)
             , Description
             , DiscountPct
             , Type
             , Category
		   , SpecialOfferID
        FROM Sales.SpecialOffer so)
   DELETE   FROM DeDup
   WHERE    RowNumber > 1;
GO


/***********************************************************************************
* Query to show duplicate records in the Sales.SpecialOffer table.
***********************************************************************************/
SELECT   Description
       , DiscountPct
       , Type
       , Category
       , RecordCount = COUNT(1)
FROM     Sales.SpecialOffer so
GROUP BY Description
       , DiscountPct
       , Type
       , Category
HAVING   COUNT(1) > 1;



/***********************************************************************************
* PART II- This demo enhances a demo found in the Windows Function.
*
* Running the following query finds the TOP 5 sales for each Sales Person.
* This script uses a ranking window function to order a Sales Persons sales by the 
* total due.  Using this ranking value we can limit to the top 3.
*
* NOTE: The SELECT FROM statement using the CTE must come immediately after the 
*       CTE definition.
***********************************************************************************/
WITH RankingSalesOrder AS 
    (SELECT FirstName
             , LastName
             , TotalDue
             , TopRankingSales = RANK()OVER(PARTITION BY SalesPersonID ORDER BY TotalDue DESC)
        FROM Sales.SalesOrderHeader soh
             JOIN HumanResources.Employee e ON e.BusinessEntityID = soh.SalesPersonID
             JOIN Person.Person p ON p.BusinessEntityID = e.BusinessEntityID
    )
    SELECT FirstName
         , LastName
         , TotalDue
    FROM RankingSalesOrder
    WHERE TopRankingSales <= 3
    ORDER BY LastName, FirstName, TotalDue DESC;
GO

/* END Common Table Expression DEMO */

